module mcodesflags
!******************************************************************************
!                                   mcodesflags 
! Module to get the significations associated with codes and flags used in
! the FM94-BUFR format
!                                   
!(Módulo para obter os significados associados a códigos e bandeiras utilizados
! no formato FM94-BUFR)
!
!********************************************************************************
! HISTORICO 201411- SHSF - acrescido valor do campo associado
 use stringflib
 use mbufr, only:get_name_mbufr
 implicit none
 private
 public tabA
 public tabCC1
 public init_mcodesflags
 public signification_mcodesflags

 type codelist
  character(len=50)::description
  integer          ::descriptor
  integer          ::code
 end type

 character(len=255)                 ::mbufr_tables
 character(len=255)                 ::mbufr_tableA
 character(len=255)                 ::mbufr_tableB
 character(len=255)                 ::mbufr_commonTableC1
 character(len=255)                 ::codeflagtables
 character(len=50),dimension(0:255) ::tabA,tabCC1
 integer,parameter                  ::nrucmax=200
 type (codelist), dimension(nrucmax)::ruc                 !list of recently used codes
 integer                            ::nruc                !Number of elements in the ruc
 contains

!--------------------------------------------------------------------------------
! initializes this module and loads the table-A and  the table-C1
!--------------------------------------------------------------------------------
!
!---------------------------------------------------------------------------------
 subroutine init_mcodesflags(path2tables)
 !{ Interface
   character(len=*),intent(in)::path2tables
 !}
 !{ local variables
   character(len=50)          ::ncod
   integer                    ::icod
   integer                    ::i
 !}

 !{ loads the TableA
    tabA(:)=""
    if(len_trim(path2tables)==0) then  
       call getenv('MBUFR_TABLES',mbufr_tables)
    else 
       mbufr_tables=path2tables
    end if
    !{ Acrescenta barra no final do diretorio local_tables, caso seja necessario
    ! Nesse processo veirifica se o diretorio contem barras do windows ou barra do linux 

    i=len_trim(mbufr_tables)
    if ((mbufr_tables(i:i)/=char(92)).and.(mbufr_tables(i:i)/="/")) then 
        if (index(mbufr_tables,char(92))>0) then 
           mbufr_tables=trim(mbufr_tables)//char(92)
        else
           mbufr_tables=trim(mbufr_tables)//"/"
        end if
    end if
	mbufr_tableA=trim(mbufr_tables)//"BufrTableA.txt"
	
	open(2,file=mbufr_tableA,status="unknown")
	 
551		read(2,'(i3,1x,a50)',end=661)icod,ncod
		tabA(icod)=ncod
		goto 551	
661		continue
    close(2)
!}

!{ loads the table-C1

	mbufr_commonTableC1=trim(mbufr_tables)//"CommonTableC1.txt"
	open(2,file=mbufr_commonTableC1,status="unknown")
	tabCC1(:)=""
553		read(2,'(i3,1x,a)',end=663)icod,ncod
		tabCC1(icod)=ncod
		goto 553
663	 continue
	close(2)
!}
!{ Reads the code and flag tables
 codeflagtables=trim(mbufr_tables)//"Code-Flag_tables.txt"
 nruc=0
 !print *,signification_mcodesflags(001007,1.0)
 
end subroutine


function signification_mcodesflags(descriptor,codefigure,associated_field)
 integer,          intent(in)::descriptor 
 real,             intent(in)::codefigure
 integer*2        ,intent(in)::associated_field

 character(len=200)                  ::signification_mcodesflags
 character(len=200),dimension(1:1000)::values
 integer                             ::i,nelements
 logical                             ::found 
 character(len=200)                  ::description
 character(len=6)                    ::cdescriptor
 write(cdescriptor,'(I6.6)')descriptor
!-------------------------------------
!Check if it is a associated field
!----------------------------------

 if (associated_field>0) then 
   write(signification_mcodesflags,'("Value of the associated field (size =",i3,"bits)")')associated_field
   return
 end if 
!-----------------------------------------------------------------
! Check if it is the case of a descriptor associated a CODE TABLES 
!-----------------------------------------------------------------
 description=get_name_mbufr(descriptor)
 signification_mcodesflags=description
 if (index(description,"CODE")==0) return

 found=.false.

 

!---------------------------------------
! Check the list of recently used codes
!---------------------------------------
   do i=1,nruc
     if(ruc(i)%descriptor==descriptor) then 
       if (ruc(i)%code==codefigure) then 
         signification_mcodesflags=trim(ruc(i)%description)//"<-"//trim(description)
         found=.true.
         exit
       end if
     end if
   end do

  if (.not. found) then 
     call read_config(20,codeflagtables,cdescriptor,values,nelements)
   !  print *,">",nelements
     do i=1,nelements,2
      if (val(values(i))==codefigure) then 
         signification_mcodesflags=trim(values(i+1))//"<-"//trim(description)
         nruc=nruc+1
         if (nruc>nrucmax) nruc=nrucmax
         ruc(nruc)%descriptor=descriptor
         ruc(nruc)%description=values(i+1)
         ruc(nruc)%code=codefigure
        ! write (*,22),ruc(nruc)%descriptor,ruc(nruc)%code,trim(ruc(nruc)%description)
        !22 format(2x,":MCODESFLAGS: Including [descriptor,code,description]=",i6.6,1x,i4,1x,a)
       end if
     end do
     close(20)
   end if

end function
end module
